#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "data_entry.h"
#include <unistd.h>
#include <libpq-fe.h>
#include <gdk/gdkkeysyms.h>
#include "pizza_utils.h"

static DataEntry *display_file_screen = NULL;
static char *the_file_contents = NULL;
static GtkText *the_text;
static GtkWidget *print_btn;
static char the_filename[256];

/*----------------------------------------------------------
::display file routines
-----------------------------------------------------------*/

enum {
	DISPLAY_FILE_TEXT,

	DISPLAY_FILE_PRINT_BTN,
	DISPLAY_FILE_QUIT_BTN
};

void hide_file_display ()
{
	if (display_file_screen)
		gtk_widget_hide (GTK_WIDGET(display_file_screen));

	if (the_file_contents)
		free (the_file_contents);
	the_file_contents = NULL;
}

void display_file_quit ()
{
	hide_file_display ();
}

void display_file_print ()
{
	char Buffer[512];

	sprintf (Buffer, "print_file %s", the_filename);
	system (Buffer);
}

char *load_file_data (char *filename)
{
	FILE *fp;
	long len;

	fp = fopen (filename, "r");
	fseek (fp, 0, SEEK_END);
	len = ftell (fp);

	if (the_file_contents)
		free (the_file_contents);

	the_file_contents = malloc (len + 1);
	the_file_contents[len] = 0;

	rewind (fp);
	fread (the_file_contents, len, 1, fp);
	fclose (fp);

	return the_file_contents;
}

static void make_display_file (char *filename)
{
	the_file_contents = load_file_data (filename);

	if (!display_file_screen)
	{
		display_file_screen = DATA_ENTRY(data_entry_new (filename, 50, 100, 0,0));

		gtk_widget_realize (GTK_WIDGET(display_file_screen));

		the_text = GTK_TEXT(gtk_text_new (NULL,NULL));
		gtk_widget_set_usize (GTK_WIDGET(the_text), 600,400);
		gtk_text_set_editable (the_text, FALSE);

		data_entry_add_widget (display_file_screen, DISPLAY_FILE_TEXT,
			GTK_WIDGET(the_text), 0,0,1,1);

		data_entry_add_label_button (display_file_screen, DISPLAY_FILE_PRINT_BTN, "Print", 
				GTK_SIGNAL_FUNC(display_file_print), BUTTON_DEFAULT);

		data_entry_add_label_button (display_file_screen, DISPLAY_FILE_QUIT_BTN, "Quit", 
				GTK_SIGNAL_FUNC(display_file_quit), BUTTON_QUIT);

		gtk_widget_realize (GTK_WIDGET(the_text));
	}

	gtk_editable_delete_text (GTK_EDITABLE(the_text),0,-1);
	gtk_text_insert (the_text, NULL,NULL,NULL, the_file_contents, strlen(the_file_contents));
	if (!GTK_WIDGET_VISIBLE(display_file_screen))
	{
		gtk_widget_show (GTK_WIDGET(display_file_screen));
	}
	gtk_widget_grab_focus (GTK_WIDGET(the_text));

	gtk_window_set_title (GTK_WINDOW(display_file_screen), filename);
	gdk_window_raise (GTK_WIDGET(display_file_screen)->window);

	strcpy (the_filename , filename);

}

void display_file (char *filename)
{
	make_display_file (filename);
	gtk_widget_hide (print_btn);
}

void display_print_file (char *filename)
{
	make_display_file (filename);
	gtk_widget_show (print_btn);
}

