#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "data_entry.h"
#include <unistd.h>
#include <libpq-fe.h>
#include <gdk/gdkkeysyms.h>
#include "pizza_utils.h"
#include "reports.h"
#include "display_file.h"

DataEntry *report_selection = NULL;
static GtkWidget *date_from, *date_to, *report_types, *report_count, *label_field;

enum {
	REPORT_TYPE,
	REPORT_DATE_FROM,
	REPORT_DATE_TO,
	REPORT_COUNT,

	REPORT_GO_BTN,
	REPORT_SHOW_LAST_BTN,
	REPORT_QUIT_BTN
};

char date_from_string [32];
char date_to_string [32];

/*----------------------------------------------------------------------------------------------------
::actual reports
----------------------------------------------------------------------------------------------------*/

static void create_sales_report ()
{
	double totals[3];
	double x;
	FILE *fp;
	int nTuples, i;
	char buffer[64];

	totals[0] = totals[1] = totals[2] = 0.0;

	fp = fopen ("sales_report.rep", "w");
	if (!fp) return;
	fprintf (fp, "Sales from %s to %s\n\n", date_from_string, date_to_string);
	fprintf (fp, "Pickups\n=======================================\n");

	res = execSQL ("select order_date,pickup_name,total_price from orders "
		"where is_pickup = 't' and order_date >= '%s' and order_date < '%s' "
		"order by order_date ",
		date_from_string, date_to_string);
	nTuples = PQntuples (res);
	for (i = 0; i < nTuples; i++)
	{
		x = atof (PQgetvalue (res,i,2));
		fprintf (fp, "%12.12s %-19.19s %s\n",
			PQgetvalue (res,i,0) + 4,
			PQgetvalue (res,i,1),
			money (x, 6));
		totals[0] += x;
	}

	PQclear (res);
	fprintf (fp, "---------------------------------------\n");
	fprintf (fp, "                     Total   %s\n\n",
		money (totals[0], 10));
	totals[2] = totals[0];

	fprintf (fp, "Deliveries\n=======================================\n");
	res = execSQL ("select o.order_date,o.custnum,c.phone,c.suburb,o.total_price "
		"from orders o, customers c "
		"where o.is_pickup = 'f' and o.order_date >= '%s' and o.order_date < '%s' "
		"and c.custnum = o.custnum "
		"order by order_date",
		date_from_string, date_to_string);
	nTuples = PQntuples (res);
	for (i = 0; i < nTuples; i++)
	{
		x = atof (PQgetvalue (res,i,4));
		sprintf (buffer, "%s %s", 
			PQgetvalue (res,i,2),
			PQgetvalue (res,i,3));
		fprintf (fp, "%12.12s %5.5s %-13.13s %s\n",
			PQgetvalue (res,i,0) + 4,
			PQgetvalue (res,i,1),
			buffer,
			money (x, 6));
		totals[1] += x;
	}
	PQclear (res);
	fprintf (fp, "---------------------------------------\n");
	fprintf (fp, "                     Total   %s\n\n", money (totals[1], 10));
	totals[2] += totals[1];
	fprintf (fp, "               Total Sales  %s\n", money (totals[2], 11));

	fclose (fp);

	display_print_file ("sales_report.rep");
}

static void create_prod_report ()
{
	double totals[2];
	FILE *fp;
	int nTuples, i;
	int nTuples2, i2;
	char *current_catd, *current_cat;
	PGresult *res2, *res3;
	float qty, price;
	float tqty, tprice;
	float ttqty, ttprice;

	fp = fopen ("prod_report.rep", "w");
	if (!fp) return;
	fprintf (fp, "Product Sales from %s to %s\n\n", date_from_string, date_to_string);

	// first create the product reports database
	res = PQexec (conn, "drop table prodrep");
	PQclear (res);
	res = execSQL ("create table prodrep (code text,qty float8, price float8)");
	PQclear (res);
	res = execSQL ("insert into prodrep (code,qty,price) "
		"select d.item_code,d.qty,d.price from orders o, ordet d "
		"where o.order_date >= '%s' and o.order_date < '%s' "
		"and d.ordernum = o.ordernum",
		date_from_string, date_to_string);
	PQclear (res);
	res = execSQL ("create index prodrepc on prodrep (code)");
	PQclear (res);

	ttqty = ttprice = 0.0;
	res = execSQL ("select code,description from category order by code");
	nTuples = PQntuples (res);
	for (i = 0; i < nTuples; i++)
	{
		current_cat = PQgetvalue (res,i,0);
		current_catd = PQgetvalue (res,i,1);
		totals[0] = 0.0;
		fprintf (fp, "%s %s\n=======================================\n", current_cat, current_catd);

		// get all products for the category
		res2 = execSQL ("select code, description from items where category = '%s' "
			"order by code", 
			current_cat);
		tqty = tprice = 0.0;
		nTuples2 = PQntuples (res2);
		for (i2 = 0; i2 < nTuples2; i2++)
		{
			res3 = execSQL ("select sum(qty), sum(price) from prodrep where code = '%s'",
				PQgetvalue (res2, i2, 0));
			qty = atof(PQgetvalue (res3, 0,0));
			price = atof(PQgetvalue (res3,0,1));

			if (qty)
			{
			fprintf (fp, "%-6.6s %-19.19s ", 
				PQgetvalue (res2, i2, 0),
				PQgetvalue (res2, i2, 1));

			fprintf (fp, "%-4.0f %s\n",
				qty,
				money (price, 7)
				);
			tqty += qty;
			tprice += price;
			}
			PQclear (res3);
		}
		PQclear (res2);

		fprintf (fp,        "---------------------------------------\n");
		fprintf (fp,        "                 Total   %6.0f %s\n\n",
			tqty, money (tprice, 7));
		ttqty += tqty;
		ttprice += tprice;
	}
	PQclear (res);
	fprintf (fp, "---------------------------------------\n");
	fprintf (fp, "       Total Sales %8.0f %s\n", ttqty, money(ttprice, 11));
	fclose (fp);
	display_print_file ("prod_report.rep");
}

static void create_daily_report ()
{
	FILE *fp;
	int nTuples, i, the_day;
	int day_count;
	char start_date[32];
	char catCodes[64];
	int num_cats;
	PGresult *res2;
	int nTuples2, ii;

	day_count = atoi(gtk_input_get_text (report_count));
	sprintf (start_date, "%s 08:00", date_from_string);
	fp = fopen ("day_report.rep", "w");
	if (!fp) return;
	fprintf (fp, "Product Sales from %s for %d days\n\n", start_date, day_count);

	// first create the product reports database
	res = PQexec (conn, "drop table dayrep");
	PQclear (res);
	res = execSQL ("create table dayrep (order_date abstime, category char,qty float8, price float8)");
	PQclear (res);
	res = execSQL ("insert into dayrep (order_date,category,qty,price) "
		"select o.order_date,i.category,d.qty,d.price "
		"from orders o,ordet d, items i "
		"where o.order_date >= '%s' and o.order_date < ('%s'::abstime + '%d day'::reltime) "
		"and d.ordernum = o.ordernum "
		"and i.code = d.item_code",
		start_date, start_date, day_count+1);
	PQclear (res);
	res = execSQL ("create index dayrepc on dayrep (order_date)");
	PQclear (res);

	// print the categories at the top
	res = execSQL ("select code, description from category order by code");
	nTuples = PQntuples (res);
	for (i = 0; i < nTuples; i++)
		fprintf (fp, "  %1.1s %s\n", PQgetvalue (res,i,0), PQgetvalue (res,i,1));
	PQclear (res);
	fprintf (fp, "\nQuantities\nDate   ");

	res = execSQL ("select code from category order by code");
	nTuples = PQntuples (res);
	num_cats = 0;
	for (i = 0; i < nTuples && i < 64; i++)
	{
		catCodes[i] = *PQgetvalue (res,i,0);
		num_cats++;
		fprintf (fp, "%5.5s ", PQgetvalue (res,i,0));
	}
	PQclear (res);

	fprintf (fp, "\n=============================================================\n");

	for (the_day = 0; the_day < day_count; the_day++)
	{
		res = execSQL ("select order_date from dayrep where "
			"order_date >= ('%s'::abstime + '%d day'::reltime) "
			"and order_date < ('%s'::abstime + '%d day'::reltime) ",
			start_date, the_day, start_date, the_day+1);
		if (!PQntuples (res))
		{
			PQclear (res);
			continue;
		}
		fprintf (fp, "%-6.6s ", PQgetvalue (res,0,0) + 4);

		// now for each category
		for (i = 0; i < num_cats; i++)
		{
			res2 = execSQL ("select sum(qty) from dayrep where "
				"order_date >= ('%s'::abstime + '%d day'::reltime) "
				"and order_date < ('%s'::abstime + '%d day'::reltime) "
				"and category = '%c'",
				start_date, the_day, start_date, the_day+1,
				catCodes[i]);
			if (PQntuples(res2))
				fprintf (fp, "%s ", integer (atof(PQgetvalue (res2,0,0)),5));
			else
				fprintf (fp, "      ");
			PQclear (res2);
		}
		fprintf (fp, "\n");
		PQclear (res);
	}

	fprintf (fp, "\n$ Value\n=============================================================\n");
	for (the_day = 0; the_day < day_count; the_day++)
	{
		res = execSQL ("select order_date from dayrep where "
			"order_date >= ('%s'::abstime + '%d day'::reltime) "
			"and order_date < ('%s'::abstime + '%d day'::reltime) ",
			start_date, the_day, start_date, the_day+1);
		if (!PQntuples (res))
		{
			PQclear (res);
			continue;
		}
		fprintf (fp, "%6.6s ", PQgetvalue (res,0,0) + 4);

		// now for each category
		for (i = 0; i < num_cats; i++)
		{
			res2 = execSQL ("select sum(price) from dayrep where "
				"order_date >= ('%s'::abstime + '%d day'::reltime) "
				"and order_date < ('%s'::abstime + '%d day'::reltime) "
				"and category = '%c'",
				start_date, the_day, start_date, the_day+1,
				catCodes[i]);
			if (PQntuples(res2))
				fprintf (fp, "%s ", integer(atof(PQgetvalue (res2,0,0)),5));
			else
				fprintf (fp, "      ");
			PQclear (res2);
		}
		fprintf (fp, "\n");
		PQclear (res);
	}

	fclose (fp);
	display_print_file ("day_report.rep");
}

static void create_week_report ()
{
	FILE *fp;
	int nTuples, i, the_day;
	int day_count;
	char start_date[32];
	char catCodes[64];
	int num_cats;
	PGresult *res2;
	int nTuples2, ii;

	day_count = atoi(gtk_input_get_text (report_count));
	sprintf (start_date, "%s 08:00", date_from_string);
	fp = fopen ("week_report.rep", "w");
	if (!fp) return;
	fprintf (fp, "Product Sales from %s for %d weeks\n\n", start_date, day_count);

	// first create the product reports database
	res = PQexec (conn, "drop table dayrep");
	PQclear (res);
	res = execSQL ("create table dayrep (order_date abstime, category char,qty float8, price float8)");
	PQclear (res);
	res = execSQL ("insert into dayrep (order_date,category,qty,price) "
		"select o.order_date,i.category,d.qty,d.price "
		"from orders o,ordet d, items i "
		"where o.order_date >= '%s' and o.order_date < ('%s'::abstime + '%d week'::reltime) "
		"and d.ordernum = o.ordernum "
		"and i.code = d.item_code",
		start_date, start_date, day_count+1);
	PQclear (res);
	res = execSQL ("create index dayrepc on dayrep (order_date)");
	PQclear (res);

	// print the categories at the top
	res = execSQL ("select code, description from category order by code");
	nTuples = PQntuples (res);
	for (i = 0; i < nTuples; i++)
		fprintf (fp, "  %1.1s %s\n", PQgetvalue (res,i,0), PQgetvalue (res,i,1));
	PQclear (res);
	fprintf (fp, "\nQuantities\nDate   ");

	res = execSQL ("select code from category order by code");
	nTuples = PQntuples (res);
	num_cats = 0;
	for (i = 0; i < nTuples && i < 64; i++)
	{
		catCodes[i] = *PQgetvalue (res,i,0);
		num_cats++;
		fprintf (fp, "%5.5s ", PQgetvalue (res,i,0));
	}
	PQclear (res);

	fprintf (fp, "\n=============================================================\n");

	for (the_day = 0; the_day < day_count; the_day++)
	{
		res = execSQL ("select order_date from dayrep where "
			"order_date >= ('%s'::abstime + '%d week'::reltime) "
			"and order_date < ('%s'::abstime + '%d week'::reltime) ",
			start_date, the_day, start_date, the_day+1);
		if (!PQntuples (res))
		{
			PQclear (res);
			continue;
		}
		fprintf (fp, "%-6.6s ", PQgetvalue (res,0,0) + 4);

		// now for each category
		for (i = 0; i < num_cats; i++)
		{
			res2 = execSQL ("select sum(qty) from dayrep where "
				"order_date >= ('%s'::abstime + '%d week'::reltime) "
				"and order_date < ('%s'::abstime + '%d week'::reltime) "
				"and category = '%c'",
				start_date, the_day, start_date, the_day+1,
				catCodes[i]);
			if (PQntuples(res2))
				fprintf (fp, "%5.5s ", PQgetvalue (res2,0,0));
			else
				fprintf (fp, "      ");
			PQclear (res2);
		}
		fprintf (fp, "\n");
		PQclear (res);
	}

	fprintf (fp, "\n$ Value\n=============================================================\n");
	for (the_day = 0; the_day < day_count; the_day++)
	{
		res = execSQL ("select order_date from dayrep where "
			"order_date >= ('%s'::abstime + '%d week'::reltime) "
			"and order_date < ('%s'::abstime + '%d week'::reltime) ",
			start_date, the_day, start_date, the_day+1);
		if (!PQntuples (res))
		{
			PQclear (res);
			continue;
		}
		fprintf (fp, "%6.6s ", PQgetvalue (res,0,0) + 4);

		// now for each category
		for (i = 0; i < num_cats; i++)
		{
			res2 = execSQL ("select sum(price) from dayrep where "
				"order_date >= ('%s'::abstime + '%d week'::reltime) "
				"and order_date < ('%s'::abstime + '%d week'::reltime) "
				"and category = '%c'",
				start_date, the_day, start_date, the_day+1,
				catCodes[i]);
			if (PQntuples(res2))
				fprintf (fp, "%s ", integer(atof(PQgetvalue (res2,0,0)),5));
			else
				fprintf (fp, "      ");
			PQclear (res2);
		}
		fprintf (fp, "\n");
		PQclear (res);
	}

	fclose (fp);
	display_print_file ("week_report.rep");
}

static int validate_dates ()
{
	char last_char;
	int dash_count;
	char *ptr;
	char *toggle;
	int has_date_to;

	toggle = data_entry_get_radio_value (report_selection, REPORT_TYPE);
	strcpy (date_from_string, gtk_input_get_text (date_from));

	if (*toggle == 'S' || *toggle == 'P')
		has_date_to = 1;
	else
		has_date_to = 0;

	if (has_date_to) 
		strcpy (date_to_string, gtk_input_get_text (date_to));

	ptr = date_from_string;
	last_char = 0;
	dash_count = 0;
	while (*ptr)
	{
		switch (*ptr)
		{
			case ' ':
				if (last_char != '-') *ptr = '-';
				break;
		}
		if ((*ptr) == '-') dash_count++;
		last_char = *ptr;
		ptr++;
	}
	if (strlen(date_from_string) && dash_count != 2)
	{
		printf ("'%s' %d\n", date_from_string,dash_count);
		message_box ("Date Format DD-Mon-YYYY\n\nWhere DD = Day\n      Mon = Month\n      YYYY = Year\n\neg: 02-Jan-1998\n");
		gtk_widget_grab_focus (date_from);
		return 0;
	}

	if (has_date_to)
	{
	ptr = date_to_string;
	last_char = 0;
	dash_count = 0;
	while (*ptr)
	{
		switch (*ptr)
		{
			case ' ':
				if (last_char != '-') *ptr = '-';
				break;
		}
		if ((*ptr) == '-') dash_count++;
		last_char = *ptr;
		ptr++;
	}
	if (strlen(date_to_string) && dash_count != 2)
	{
		printf ("2 '%s' %d\n", date_to_string,dash_count);
		message_box ("Date Format DD-Mon-YYYY\n\nWhere DD = Day\n      Mon = Month\n      YYYY = Year\n\neg: 02-Jan-1998\n");
		gtk_widget_grab_focus (date_from);
		return 0;
	}
	}
	return 1;
}

static void set_report_labels ()
{
	switch (*data_entry_get_radio_value (report_selection, REPORT_TYPE))
	{
		case 'D':
			gtk_widget_show (report_count);
			gtk_widget_hide (date_to);
			gtk_label_set (label_field, "No of Days:");
			break;
		case 'W':
			gtk_widget_show (report_count);
			gtk_widget_hide (date_to);
			gtk_label_set (label_field, "No of Weeks:");
			break;
		case 'S':
		case 'P':
		default:
			gtk_widget_hide (report_count);
			gtk_widget_show (date_to);
			gtk_label_set (label_field, "Date To:");
			break;
	}
}

static void report_go ()
{
	printf ("doing report %s\n", data_entry_get_radio_value (report_selection, REPORT_TYPE));
	if (!validate_dates ())
		return;
	switch (*data_entry_get_radio_value (report_selection, REPORT_TYPE))
	{
		case 'S':
			create_sales_report ();
			break;
		case 'D':
			create_daily_report ();
			break;
		case 'P':
			create_prod_report ();
			break;
		case 'W':
			create_week_report ();
			break;
		default:
			break;
	}
}

static void report_last ()
{
	switch (*data_entry_get_radio_value (report_selection, REPORT_TYPE))
	{
		case 'S':
			display_print_file ("sales_report.rep");
			break;
		case 'D':
			display_print_file ("day_report.rep");
			break;
		case 'P':
			display_print_file ("prod_report.rep");
			break;
		case 'W':
			display_print_file ("week_report.rep");
			break;
		default:
			break;
	}
}

static void report_quit ()
{
	gtk_widget_hide (GTK_WIDGET(report_selection));
}

/*----------------------------------------------------------------------------------------------------
::do_reports
----------------------------------------------------------------------------------------------------*/

void do_reports ()
{
	GtkWidget *rb;

	if (!report_selection)
	{
	report_selection = DATA_ENTRY(data_entry_new ("Report Selection", 20, 20, 0,0));

	report_types = data_entry_add_radio_hbox (report_selection, REPORT_TYPE, 0,0,10,1);
	rb = data_entry_add_to_radio (report_selection, report_types, "S", "Sales");
    gtk_signal_connect (GTK_OBJECT (rb), "toggled",
		            GTK_SIGNAL_FUNC(set_report_labels), NULL);
	rb = data_entry_add_to_radio (report_selection, report_types, "P", "Product");
    gtk_signal_connect (GTK_OBJECT (rb), "toggled",
		            GTK_SIGNAL_FUNC(set_report_labels), NULL);
	rb = data_entry_add_to_radio (report_selection, report_types, "D", "Daily");
    gtk_signal_connect (GTK_OBJECT (rb), "toggled",
		            GTK_SIGNAL_FUNC(set_report_labels), NULL);
	rb = data_entry_add_to_radio (report_selection, report_types, "W", "Weekly");
    gtk_signal_connect (GTK_OBJECT (rb), "toggled",
		            GTK_SIGNAL_FUNC(set_report_labels), NULL);

	date_from = GTK_INPUT(data_entry_add_field (report_selection, REPORT_DATE_FROM, 
		"Date From:", 16, INPUT_UPPER, 1,1,2,1));
	label_field = data_entry_add_widget (report_selection, -1, gtk_label_new ("Date To:"),
		0,2,1,1);
	date_to = GTK_INPUT(data_entry_add_field (report_selection, REPORT_DATE_TO, 
		NULL, 16, INPUT_UPPER, 1,2,2,1));
	report_count = data_entry_add_field (report_selection, REPORT_COUNT,
		NULL, 4, INPUT_UPPER, 1,2,1,1);
	gtk_widget_hide (report_count);

	data_entry_add_label_button (report_selection, REPORT_GO_BTN, "Go !", 
		GTK_SIGNAL_FUNC(report_go), BUTTON_DEFAULT);
	data_entry_add_label_button (report_selection, REPORT_SHOW_LAST_BTN, "Show Last", 
		GTK_SIGNAL_FUNC(report_last), BUTTON_NORMAL);
	data_entry_add_label_button (report_selection, REPORT_QUIT_BTN, "Quit", 
		GTK_SIGNAL_FUNC(report_quit), BUTTON_QUIT);
	}


	if (!GTK_WIDGET_VISIBLE (report_selection))
		gtk_widget_show (GTK_WIDGET(report_selection));
	set_report_labels ();
	gtk_widget_grab_focus (GTK_WIDGET(date_from));
}
